FFTW 3.3.4: Dynamic Arrays in C-The Wrong Way

Previous: Dynamic Arrays in C, Up: Multi-dimensional Array Format [Contents][Index]

#### 3.2.5 Dynamic Arrays in C—The Wrong Way

A different method for allocating multi-dimensional arrays in C is often suggested that is incompatible with FFTW: using it will cause FFTW to die a painful death. We discuss the technique here, however, because it is so commonly known and used. This method is to create arrays of pointers of arrays of pointers of …etcetera. For example, the analogue in this method to the example above is:

    int i,j;
    fftw_complex ***a_bad_array;  /* another way to make a 5x12x27 array */

    a_bad_array = (fftw_complex ***) malloc(5 * sizeof(fftw_complex **));
    for (i = 0; i < 5; ++i) {
         a_bad_array[i] =
            (fftw_complex **) malloc(12 * sizeof(fftw_complex *));
         for (j = 0; j < 12; ++j)
              a_bad_array[i][j] =
                    (fftw_complex *) malloc(27 * sizeof(fftw_complex));
    }

As you can see, this sort of array is inconvenient to allocate (and deallocate). On the other hand, it has the advantage that the (i,j,k)-th element can be referenced simply by `a_bad_array[i][j][k]`.

If you like this technique and want to maximize convenience in accessing the array, but still want to pass the array to FFTW, you can use a hybrid method. Allocate the array as one contiguous block, but also declare an array of arrays of pointers that point to appropriate places in the block. That sort of trick is beyond the scope of this documentation; for more information on multi-dimensional arrays in C, see the `comp.lang.c` FAQ.